-- 7 segment LED Display Module
-- 47ZOgLED_Ci~bN_郂W[
-- led_enable͊eVZOgLED_邩ǂBPœ_
-- Spartan3 f{[hp

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity SsegDisp is
	generic(
		clk_frequency : integer := 25000 -- KHz
	);
	port(
		clk, reset : in std_logic;
		dot : in std_logic_vector(3 downto 0);
		led_enable : in std_logic_vector(3 downto 0);
		binary3 : in std_logic_vector(3 downto 0); -- AN3
		binary2 : in std_logic_vector(3 downto 0); -- AN2
		binary1 : in std_logic_vector(3 downto 0); -- AN1
		binary0 : in std_logic_vector(3 downto 0); -- AN0
		sw : in std_logic_vector(4 downto 0);
		A_n : out std_logic;
		B_n : out std_logic;
		C_n : out std_logic;
		D_n : out std_logic;
		E_n : out std_logic;
		F_n : out std_logic;
		G_n : out std_logic;
		DP_n : out std_logic;
		AN_n : out std_logic_vector(3 downto 0)
	);
end SsegDisp;

architecture RTL of SsegDisp is
component B27segDec
	port(
		binary : in std_logic_vector(3 downto 0);
		enable : in std_logic;
		segdecout : out std_logic_vector(0 to 6) -- a,b,c,d,e,f,g segment without dot.
	);
end component;
component FreqDiv
	generic(
		clk_frequency : integer := 5000 -- KHz
	);
	port(
		clk, reset : in std_logic;
		lighting_ena : out std_logic
	);
end component;
signal ledno : std_logic_vector(5 downto 0);
signal lighting_ena : std_logic;
signal decout : std_logic_vector(6 downto 0);
signal binary : std_logic_vector(3 downto 0);
signal logic1 : std_logic;
signal led_ena : std_logic;
signal nLEDOE : std_logic_vector(3 downto 0);
signal nLEDDB : std_logic_vector(6 downto 0);
signal logic0 : std_logic;
begin
	logic0 <= '0';

	-- 2rbgEoCiJE^
	process(reset, clk) begin
		if reset='1' then
			ledno <= (others => '0');
		elsif clk'event and clk='1' then
			if lighting_ena = '1' then
				if ledno=('0'&sw)+3 then
					ledno <= (others => '0');
				else
					ledno <= ledno + 1;
				end if;
			end if;
		end if;
	end process;
	
	-- \LED肷
	nLEDOE(0) <= '0' when ledno="000000" else '1';
	nLEDOE(1) <= '0' when ledno="000001" else '1';
	nLEDOE(2) <= '0' when ledno="000010" else '1';
	nLEDOE(3) <= '0' when ledno="000011" else '1';
	
	-- LEDtbVg
	FreqDivi : FreqDiv generic map(
		clk_frequency => clk_frequency
	)port map(
		clk => clk,
		reset => reset,
		lighting_ena => lighting_ena
	);
	
	-- _Ci~bN_p}`vNT
	process(ledno, binary0, binary1, binary2, binary3, dot, led_enable) begin
		case ledno is
		when "000000" =>
			binary <= binary0;
			led_ena <= led_enable(0);
			DP_n <= not dot(0);
		when "000001" =>
			binary <= binary1;
			led_ena <= led_enable(1);
			DP_n <= not dot(1);
		when "000010" =>
			binary <= binary2;
			led_ena <= led_enable(2);
			DP_n <= not dot(2);
		when "000011" =>
			binary <= binary3;
			led_ena <= led_enable(3);
			DP_n <= not dot(3);
		when others =>
			binary <= binary3;
			led_ena <= logic0;
			DP_n <= logic0;
		end case;
	end process;
	
	-- 7ZOgLEDfR[_
	B27segDeci : B27segDec port map(
		binary => binary,
		enable => led_ena,
		segdecout => decout
	);
	nLEDDB <= not decout;
	A_n <= nLEDDB(6);
	B_n <= nLEDDB(5);
	C_n <= nLEDDB(4);
	D_n <= nLEDDB(3);
	E_n <= nLEDDB(2);
	F_n <= nLEDDB(1);
	G_n <= nLEDDB(0);
	AN_n <= nLEDOE;
end RTL;
